// adaptation of demonstration : http://gmaps-samples-v3.googlecode.com/svn/trunk/distancematrix/london.html
unit UMainMatrix;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, ECMaps, Grids, StdCtrls;

type
  TFormDemoMatrix = class(TForm)
    Matrix: TStringGrid;
    map: TECMap;
    TravelMode: TComboBox;
    UnitSystem: TComboBox;
    procedure FormCreate(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure mapLoad(Sender: TObject);
    procedure mapDistanceMatrix(Sender: TObject);
    procedure MatrixClick(Sender: TObject);
    procedure mapRouteChange(sender: TObject; const idRoute: Integer;
      const NewRoute: Boolean);
    procedure MatrixMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure MatrixDrawCell(Sender: TObject; aCol, aRow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure TravelModeClick(Sender: TObject);
    procedure UnitSystemClick(Sender: TObject);
    procedure mapResize(Sender: TObject);
  private
    { Dclarations prives }
  public
    { Dclarations publiques }
    procedure UpdateMatrix;
  end;

var
  FormDemoMatrix: TFormDemoMatrix;

implementation

{$R *.DFM}

procedure TFormDemoMatrix.FormCreate(Sender: TObject);
begin
 matrix.ColCount := 6;
 matrix.RowCount := 8;


 matrix.Cells[0,1] := 'Euston';
 matrix.Cells[0,2] := 'Kings Cross';
 matrix.Cells[0,3] := 'Liverpool St';
 matrix.Cells[0,4] := 'Paddington';
 matrix.Cells[0,5] := 'St. Pancras';
 matrix.Cells[0,6] := 'Victoria';
 matrix.Cells[0,7] := 'Waterloo';

 matrix.Cells[1,0] := 'Buckingham Palace';
 matrix.Cells[2,0] := 'Houses of Parliament';
 matrix.Cells[3,0] := 'Tower of London';
 matrix.Cells[4,0] := 'Trafalgar Square';
 matrix.Cells[5,0] := 'Westminster Abbey';
end;

procedure TFormDemoMatrix.FormResize(Sender: TObject);
var i,l,lt:integer;
begin

  l := (Self.clientWidth) div matrix.ColCount;

 lt := 0;

 for i:=0 to Matrix.ColCount-2 do
  begin
   Matrix.ColWidths[i] := l;
   lt := lt + l;
  end;

  Matrix.ColWidths[Matrix.ColCount-1] := Self.clientwidth - lt ;

 matrix.Height   := (matrix.RowCount)* matrix.DefaultRowHeight;

 map.height      := self.ClientHeight - matrix.Height;
 map.width       := self.ClientWidth;
 
end;

procedure TFormDemoMatrix.UpdateMatrix;
var origins,destinations:string;
    i:integer;
begin

 map.DistanceMatrix.Origins.clear;
 map.DistanceMatrix.Destinations.clear;

 for i:=1 to 7 do
   map.DistanceMatrix.Origins.add( Matrix.Cells[0,i]+' Station, London, UK');

 for i:=1 to 5 do
   map.DistanceMatrix.Destinations.add( Matrix.Cells[i,0]+', London, UK' );

 map.DistanceMatrix.Update;


end;


// when map load update matrix
procedure TFormDemoMatrix.mapLoad(Sender: TObject);
begin

 map.DistanceMatrix.AvoidHighWays := true;
 map.DistanceMatrix.AvoidTolls    := false;

 UpDateMatrix;


 TravelMode.itemindex := 0;
 UnitSystem.itemindex := 0;
 
end;


// event fired when matrix is ok, call by map.DistanceMatrix.Update
procedure TFormDemoMatrix.mapDistanceMatrix(Sender: TObject);
var iOrigine,iDestination,Rows,Cols:integer;
begin
 if map.DistanceMatrix.count=0 then exit;

 Rows := Matrix.RowCount - 1;
 Cols := Matrix.ColCount - 1;


  for iOrigine:=0 to rows do
    for iDestination:=0 to cols do
      Matrix.Cells[1+iDestination,1+iOrigine] := map.DistanceMatrix[iOrigine,iDestination].distanceText+
                                                 #13#10+
                                                 map.DistanceMatrix[iOrigine,iDestination].durationText;

end;


// show route
procedure TFormDemoMatrix.MatrixClick(Sender: TObject);
var dmItem:TECDistanceMatrixItem;
begin
 if (Matrix.col>0)and(Matrix.row>0) then
 begin

   map.Routes.clear;

   dmItem := map.distanceMatrix[matrix.row-1,matrix.col-1];

   if assigned(dmItem) then
      map.Routes.AddByAdr('',dmItem.Origin,dmItem.Destination,false,nil);
  
 end;
end;


// adapt route to size of map
procedure TFormDemoMatrix.mapRouteChange(sender: TObject;
  const idRoute: Integer; const NewRoute: Boolean);
begin

 map.fitBounds(map.Routes[idRoute].SouthWestLatitude,map.Routes[idroute].SouthWestLongitude,
                map.Routes[idRoute].NorthEastLatitude,map.Routes[idRoute].NorthEastLongitude);

end;



// change route when mouse move on a new cell
procedure TFormDemoMatrix.MatrixMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
var acol,arow:integer;
begin

  Matrix.MouseToCell(X, Y, ACol, ARow);

  if (ACol<1) or (ARow<1) then exit;

  if (ACol<>Matrix.Col) or
     (aRow<>Matrix.Row) then
  begin
   Matrix.col := aCol;
   Matrix.row := aRow;
   MatrixClick(Matrix);
  end;

  Matrix.LeftCol := 1;
  Matrix.TopRow  := 1;
end;



// draw TStringGrid
procedure TFormDemoMatrix.MatrixDrawCell(Sender: TObject; aCol,
  aRow: Integer; Rect: TRect; State: TGridDrawState);
begin
With Sender As TStringGrid Do With Canvas Do
  Begin


   
    If (gdFixed in State) and ((ACol>0)or(ARow>0)) Then
      Brush.Color := clBtnFace
    Else If gdSelected In State Then
            Brush.Color := clNavy
          Else Brush.Color := clWhite;


    FillRect(Rect);


    Pen.Color := cl3DLight;

    Pen.width := 1;

    Moveto(rect.left,rect.bottom-1);
    Lineto(rect.right-1,rect.bottom-1);
    LineTo(rect.right-1,rect.top-1);


    if (ACol=0)or(ARow=0) then
     Canvas.Font.Style := [fsBold]
    else
     Canvas.font.Style := [];

    If gdSelected In State Then
      SetTextColor(Canvas.Handle,clWhite)
    Else SetTextColor(Canvas.Handle,clBlack);

    if (ACol=0)or(ARow=0) then

     DrawText(Canvas.Handle, PChar(Cells[aCol,aRow]), -1, Rect ,
              DT_CENTER or DT_VCENTER or DT_NOPREFIX or DT_SINGLELINE )

    else
     begin
      rect.top := rect.top + 4;
      DrawText(Canvas.Handle, PChar(Cells[aCol,aRow]), -1, rect ,
              DT_CENTER or DT_NOPREFIX or DT_WORDBREAK ) ;
     end;




  End;
end;


// change TravelMode
procedure TFormDemoMatrix.TravelModeClick(Sender: TObject);
begin

 case TravelMode.itemindex of
  0 : map.DistanceMatrix.TravelMode := tmDriving;
  1 : map.DistanceMatrix.TravelMode := tmWalking;
 end;

 MatrixClick(matrix);

end;


// change UnitSystem
procedure TFormDemoMatrix.UnitSystemClick(Sender: TObject);
begin

  case UnitSystem.itemindex of
  0 : map.DistanceMatrix.UnitSystem := usMetric;
  1 : map.DistanceMatrix.UnitSystem := usImperial;
 end;

 MatrixClick(matrix);

end;

procedure TFormDemoMatrix.mapResize(Sender: TObject);
begin
 travelMode.Top := map.top + map.Height - 70;
 travelMode.left:= map.left+ map.width  - travelMode.width - 10;

 unitsystem.Top := map.top + map.Height - 40;
 unitsystem.left:= map.left+ map.width  - unitsystem.width - 10;
end;

end.
