unit UMainNativeRoute;
{$I Delphi_Versions.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Spin, Buttons, ECMaps, ExtCtrls, OleCtrls,
  ComCtrls, TeeProcs, TeEngine, Chart, Series, ShDocVw, MSHTML,
  uecNativeMapControl,
  uecGeoLocalise, activex, uecNativeshape,
  // add uecNativeLabelLayer for the label triangles animated
  uecNativeLabelLayer,
  // add uecEditNativeLine for recalculate a route interactively
  uecEditNativeLine, UECMapUtil;

type
  TFDemoNativeRoute = class(TForm)
    PanelStartEnd: TPanel;
    cbStart: TComboBox;
    lbstart: TLabel;
    Label1: TLabel;
    cbDestination: TComboBox;
    Panel2: TPanel;
    RouteAdd: TButton;
    Colordialog: TColorDialog;
    Panel1: TPanel;
    routes: TComboBox;
    pcInfo: TPageControl;
    TabTrip: TTabSheet;
    Panel3: TPanel;
    Panel13: TPanel;
    WebBrowser: TWebBrowser;
    Splitter1: TSplitter;
    TabProfil: TTabSheet;
    Chart: TChart;
    pnCalculAltitude: TPanel;
    progressAltitude: TProgressBar;
    Label2: TLabel;
    Panel4: TPanel;
    NextPageElevation: TSpeedButton;
    Panel5: TPanel;
    PrevPageElevation: TSpeedButton;
    Panel6: TPanel;
    btAbortAlt: TButton;
    map: TECNativeMap;
    rbDriving: TRadioButton;
    rbBicycle: TRadioButton;
    rbWalking: TRadioButton;
    RouteEValue: TSpinEdit;
    RouteColor: TPanel;

    procedure RouteAddClick(Sender: TObject);

    procedure routesChange(Sender: TObject);

    procedure RouteColorClick(Sender: TObject);

    procedure cbStartChange(Sender: TObject);

    procedure pcInfoChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Panel5Resize(Sender: TObject);
    procedure Panel4Resize(Sender: TObject);
    procedure NextPageElevationClick(Sender: TObject);
    procedure PrevPageElevationClick(Sender: TObject);
    procedure ChartClickSeries(Sender: TCustomChart; Series: TChartSeries;
      ValueIndex: Integer; Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer);
{$IFDEF D5_OR_HIGHER}
    procedure WebBrowserBeforeNavigate2
      (Sender: TObject; const pDisp: IDispatch; var URL, Flags,
      TargetFrameName, PostData, Headers: OleVariant; var Cancel: WordBool);
{$ELSE}
    procedure WebBrowserBeforeNavigate2(Sender: TObject; pDisp: IDispatch;
      var URL, Flags, TargetFrameName, PostData, Headers: OleVariant;
      var Cancel: WordBool);
{$ENDIF}
    procedure btAbortAltClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);

    procedure FormActivate(Sender: TObject);

    procedure RouteEValueChange(Sender: TObject);
    procedure mapRoutePath(Sender: TObject; const params: string;
      routePath: TECRoutePath);
    procedure rbDrivingClick(Sender: TObject);

  private
    { Dclarations prives }

    idMarker: Integer;

    FecNativeLineToRoute: TecNativeLineToRoute;

    // labels layer see uecNativeLabelLayer
    FLabelLayer : TecNativeLabelLayer;

    procedure doShowDataRoute(const id: Integer);
    procedure doUpdateDataRoute(const id: Integer);
    procedure doUpdateElevation;
    procedure doShowAltitude(const Polyline: TECShapeline);
    procedure doOnGetAltitude(Sender: TECShapeline; const Total: Integer;
      var Cancel: boolean);

    procedure doEndAnimationMove(sender : TObject);
    procedure doOnMoveTriangle(sender : TObject;const Item : TECShape; var cancel:boolean);

  public
    { Dclarations publiques }
  end;

var
  FDemoNativeRoute: TFDemoNativeRoute;

implementation

{$R *.DFM}



const
  HTML_BODY_TEMPLATE = '<html><style type="text/css">' +
    'body{ background: #fff;color: #000000;overflow-x:hidden;' +
    'margin : 0;' + 'border-style :none;' + '}' +
    'div {margin:0;padding:5px;width:100%;overflow-x:hidden}' +
    'div a {text-decoration:none;font: 12pt verdana;color: #111;' +
    'margin : 0;' + 'display:block}' +
    '.km {color:#4FB2FF;font-size:x-small; text-align:right}' +
    '.odd{background: #eee;color: #000000;}' +

    '</style>' +

    '<body>';

  { *
    event OnBeforeNavigate2

    fired when click on trip information list

    url = xxx#id_item_list_information

  }

procedure TFDemoNativeRoute.WebBrowserBeforeNavigate2(Sender: TObject;
{$IFDEF D5_OR_HIGHER}const
  {$ENDIF} pDisp: IDispatch; var URL, Flags, TargetFrameName, PostData,
  Headers: OleVariant; var Cancel: WordBool);
var
  i: Integer;
  link: string;
begin

  { get only id }
  i := pos('#', URL);

  if i < 1 then
    exit;

  i := SecureStrToInt(copy(URL, i + 1, length(URL)), -1);

  if i < 0 then
    exit;

  Cancel := true;

  if not assigned(map.shapes.lines[routes.ItemIndex]) then
    exit;

  map.shapes.markers[idMarker].setPosition
    (map.shapes.lines[routes.ItemIndex].Path[i].lat,
    map.shapes.lines[routes.ItemIndex].Path[i].lng);

  map.Panto(map.shapes.markers[idMarker].latitude, map.shapes.markers[idMarker]
      .longitude);

end;

{ *
  Add new route
}
procedure TFDemoNativeRoute.RouteAddClick(Sender: TObject);
var
  lat1, lng1, lat2, lng2: double;
  title: string;
  RouteType: TMQRouteType;
begin

  if rbWalking.checked then
    RouteType := rtPedestrian
  else if rbBicycle.checked then
    RouteType := rtBicycle
  else
    RouteType := rtFastest;

  RouteAdd.enabled := false;

  // When the route calculation is completed, the event RoutePath will be called
  map.GetAsyncRoutePathByAdress(cbStart.Text, cbDestination.Text, RouteType);

  title := cbStart.Text + ' > ' + cbDestination.Text + ' > ' + rtToStr
    (RouteType);

  routes.ItemIndex := routes.items.add(title);

end;

// in response at map.GetAsyncRoutePathByAdress
procedure TFDemoNativeRoute.mapRoutePath(Sender: TObject; const params: string;
  routePath: TECRoutePath);
var
  Line: TECShapeline;

  moving : TECAnimationMoveOnPath  ;
  i      : integer;
begin

  RouteAdd.enabled := true;

  if routePath.distance = 0 then
  begin
    ShowMessage('Route not found !' + #13#10 + 'Try this format:' + #13#10 +
        #13#10 + 'Street, Town, Country');
    exit;
  end;

  // addRoute create a TECShapeLine
  Line := map.addRoute(routePath);
  // roads are managed as part TECShapeLine
  Line.color := RouteColor.color;
  Line.weight := RouteEValue.value;
  // show intermediate points
  Line.ShowText := true;

  // Create an animation to move a triangle along the road
  //
  // starting from km 0
  // speed 150 km / h
  moving := TECAnimationMoveOnPath.create(line,0,150);

 // create triangle
  i := map.shapes.pois.Add(0,0);

  map.shapes.pois[i].POIShape := poiTriangle;
  map.shapes.pois[i].width := 18;
  map.shapes.pois[i].height:= 24;

  map.shapes.pois[i].color := GetShadowColorBy(RouteColor.color,32);

  // to adjust the hint with every movement
  map.shapes.pois[i].OnShapeMove := doOnMoveTriangle;

  // ajusted the relative position and size of label
  FLabelLayer.YAnchor := 10;
  FLabelLayer.Width   := 150;

  // add label to triangle
  FLabelLayer.Add(map.shapes.pois[i]);

  map.shapes.pois[i].animation := moving;

 // event fired when arrived
  moving.OnDriveUp := doEndAnimationMove;

  // start animation
  moving.stop := false;

  //  directing the tip of the triangle in the direction of travel
  moving.heading := true;

  doShowDataRoute(Line.indexof);

end;

// fired when triangle move
procedure TFDemoNativeRoute.doOnMoveTriangle(sender : TObject;const Item : TECShape; var cancel:boolean);
var lat,lng : string;
begin
 if assigned(item) then
 begin

  lat := DoubleToStrDigit(item.Latitude,4);
  Lng := DoubleToStrDigit(item.Longitude,4);

  while length(lat)<7 do  lat := ' '+lat;
  while length(lng)<7 do  lng := ' '+lng;


  item.Hint := '<font face="Courier New" bkcolor=000 color=ffffff>Lat: </font><font face="Courier New" bkcolor=000 color=ff0000>'+Lat+'</font><br="-7">'+
               '<font face="Courier New" bkcolor=000 color=ffffff>Lng: </font><font face="Courier New" bkcolor=000 color=ff0000>'+Lng+'</font>';
 end;
end;

// reverse the direction of travel when one reaches the end of term
procedure TFDemoNativeRoute.doEndAnimationMove(sender : TObject);
begin
     TECAnimationMoveOnPath(TECShape(sender).animation).ComeBack := not TECAnimationMoveOnPath(TECShape(sender).animation).ComeBack;
     TECAnimationMoveOnPath(TECShape(sender).animation).Stop := false;
end;

{ *
  select new route
}
procedure TFDemoNativeRoute.routesChange(Sender: TObject);
begin
  doShowDataRoute(routes.ItemIndex);

end;

{ *
  update the screen with the properties of the selected route
}
procedure TFDemoNativeRoute.doShowDataRoute(const id: Integer);
var
  i, j, wpcount: Integer;
  s, html: string;

  odd: boolean;

  l: TStringList;
begin

  if id < 0 then
    exit;

  s := routes.Text;

  cbStart.Text := trim(strtoken(s, '>'));
  cbDestination.Text := trim(strtoken(s, '>'));

  case StrToRT(trim(s)) of

    rtPedestrian:
      rbWalking.checked := true;

    rtFastest:
      rbDriving.checked := true;

    rtBicycle:
      rbBicycle.checked := true;
  end;

  routes.ItemIndex := id;



  RouteColor.color := map.shapes.lines[id].color;
  RouteEValue.value := map.shapes.lines[id].weight;

  // instructions step by step
  html := HTML_BODY_TEMPLATE;
  odd := false;
  j := map.shapes.lines[id].count - 1;

  for i := 0 to j do
  begin

    s := map.shapes.lines[id].Path[i].Text;

    // all steps have no instructions

    if s = '' then
      continue;

    html := html + '<div id="step' + inttostr(i) + '"';

    if odd then
      html := html + ' class="odd"';

    odd := not odd;

    // distance is in meters, here in km

    html := html + '><a href="#' + inttostr(i)
      + '">' + s + '<div class="km">' + format
      ('%8.2f km', [map.shapes.lines[id].Path[i].distance / 1000])
      + '</div></a>' + '</div>';
  end;

  html := html + '<div ';

  if odd then
    html := html + ' class="odd"';
  odd := not odd;

  html := html + '>' + format(
    'Distance : <b>%.2f km</b><br>Estimated time : <b>' + TimeToStr
      (map.shapes.lines[id].duration / 24 / 3600),
    [map.shapes.lines[id].distance]) + '</b></div>';

  html := html + '</body></html>';

  l := TStringList.create;
  l.Text := html;
  l.savetofile(application.exename + '.htm');
  WebBrowser.Navigate(application.exename + '.htm');

  // WebBrowserLoadHtml declared in UECMapUtil
  // WebBrowserLoadHtml(WebBrowser,html);


   // move marker at start route
  map.shapes.markers[idMarker].setPosition
    (map.shapes.lines[id].Path[0].lat, map.shapes.lines[id].Path[0].lng);


//  if FecNativeLineToRoute.Line = map.shapes.lines[id] then exit;

   // recalculate a route interactively
  FecNativeLineToRoute.Line := map.shapes.lines[id];



  map.PanTo(map.shapes.markers[idMarker].latitude, map.shapes.markers[idMarker]
      .longitude);

  pcInfoChange(nil);

end;

{ *
  update properties without recalculating the route
}
procedure TFDemoNativeRoute.doUpdateDataRoute(const id: Integer);
begin
  if id < 0 then
    exit;

  map.shapes.lines[id].color := RouteColor.color;

  map.shapes.lines[id].weight := RouteEValue.value;

end;

{ *
  select color
}
procedure TFDemoNativeRoute.RouteColorClick(Sender: TObject);
begin
  if Colordialog.execute then
  begin (Sender as TPanel)
    .color := Colordialog.color;
    if (routes.ItemIndex > -1) then
      map.shapes.lines[routes.ItemIndex].color := RouteColor.color

  end;
end;

procedure TFDemoNativeRoute.RouteEValueChange(Sender: TObject);
begin
  if (routes.ItemIndex > -1) then
    map.shapes.lines[routes.ItemIndex].weight := RouteEValue.value;
end;

procedure TFDemoNativeRoute.cbStartChange(Sender: TObject);
begin
  RouteAdd.enabled := (cbStart.Text <> '') and (cbDestination.Text <> '');
end;

{ *
  event fired by getAltitudes

  @param Sender TECShapeLine
  @param Total number of altitude's point calculated
  @cancel flag for abort calcul
}
procedure TFDemoNativeRoute.doOnGetAltitude(Sender: TECShapeline;
  const Total: Integer; var Cancel: boolean);
begin
  progressAltitude.Position := Total;
  Cancel := btAbortAlt.tag = -1;

  // tag = 0 for recalculate altitude
  if Cancel then
    Sender.tag := 0;

end;

procedure TFDemoNativeRoute.pcInfoChange(Sender: TObject);
begin
  doUpdateElevation;
end;

procedure TFDemoNativeRoute.doUpdateElevation;
var
  Polyline: TECShapeline;

begin
  if pcInfo.ActivePage <> TabProfil then
    exit;

  if not assigned(map.shapes.lines[routes.ItemIndex]) then
    exit;

  application.ProcessMessages;

  Polyline := nil;

  Polyline := map.shapes.lines[routes.ItemIndex];

  if assigned(Polyline) and (Polyline.tag = 0) then
  begin

    progressAltitude.min := 0;
    progressAltitude.max := Polyline.count;
    progressAltitude.Position := 0;

    btAbortAlt.tag := 0;

    pnCalculAltitude.visible := true;

    application.ProcessMessages;

    Polyline.tag := 1;

    Polyline.getAltitudes(doOnGetAltitude);

    pnCalculAltitude.visible := false;

  end;

  doShowAltitude(Polyline);

  application.ProcessMessages;

end;

procedure TFDemoNativeRoute.doShowAltitude(const Polyline: TECShapeline);
var
  i: Integer;
  Altitude, Km: double;
begin

  if not assigned(Polyline) then
    exit;

  Chart.Series[0].clear;
  Chart.title.Text[0] := '';
  Chart.BottomAxis.Automatic := true;

  Chart.Series[0].addXY(Km, Polyline.Path[0].Alt, '', Polyline.color);
  Km := 0;

  for i := 1 to Polyline.count - 1 do
  begin
    Km := Km + DistanceFrom(Polyline.Path[i - 1].latitude,
      Polyline.Path[i - 1].longitude, Polyline.Path[i].latitude,
      Polyline.Path[i].longitude);

    Altitude := Polyline.Path[i].Alt;

    if Altitude < 0 then
      Altitude := 0;

    Chart.Series[0].addXY(Km, Altitude, '', Polyline.color);

  end;
end;

procedure TFDemoNativeRoute.FormActivate(Sender: TObject);
begin
  show;
end;

procedure TFDemoNativeRoute.FormClose
  (Sender: TObject; var Action: TCloseAction);
begin
  FecNativeLineToRoute.free;
  FLabelLayer.free;

  Action := caFree;
end;

procedure TFDemoNativeRoute.FormCreate(Sender: TObject);
begin

  // class for edit line
  FecNativeLineToRoute          := TecNativeLineToRoute.create;
  FecNativeLineToRoute.OnChange := routesChange;

  cbStart.tag := -1;
  cbDestination.tag := -1;

  with Chart do
  begin
    AddSeries(TAreaSeries.create(Chart));
    title.Text.clear;
    Chart.title.Text.add('');
    Legend.visible := false;
  end;

  map.LocalCache := extractfilepath(application.exename) + 'cache';

  { new marker }
  idMarker := map.shapes.markers.add(map.latitude, map.longitude);
  { load new icon }
  map.shapes.markers[idMarker].filename :=
    'http://google-maps-icons.googlecode.com/files/car.png';

  map.shapes.markers[idMarker].zindex := 1; // for show over the line

  pcInfo.ActivePage := TabTrip;

  // label triangles animated
 FLabelLayer := TecNativeLabelLayer.create(map,'labels');
 FLabelLayer.visible := true;


end;

procedure TFDemoNativeRoute.Panel5Resize(Sender: TObject);
begin
  PrevPageElevation.top := (Panel5.height div 2) - 20;
end;

procedure TFDemoNativeRoute.Panel4Resize(Sender: TObject);
begin
  NextPageElevation.top := (Panel4.height div 2) - 20;
end;

procedure TFDemoNativeRoute.NextPageElevationClick(Sender: TObject);
begin
  Chart.Nextpage;
end;

procedure TFDemoNativeRoute.PrevPageElevationClick(Sender: TObject);
begin
  Chart.PreviousPage;
end;

procedure TFDemoNativeRoute.rbDrivingClick(Sender: TObject);
var title:string;
begin

  if rbWalking.checked then

    FecNativeLineToRoute.RouteType := rtPedestrian

  else

    if rbDriving.checked then

    FecNativeLineToRoute.RouteType := rtFastest

  else

    FecNativeLineToRoute.RouteType := rtBicycle;


  title := cbStart.Text + ' > ' + cbDestination.Text + ' > ' + rtToStr
    (FecNativeLineToRoute.RouteType);

  if routes.ItemIndex>-1 then
  begin
   routes.items[routes.ItemIndex] := title;
  end;


end;

procedure TFDemoNativeRoute.ChartClickSeries(Sender: TCustomChart;
  Series: TChartSeries; ValueIndex: Integer; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);

var
  Polyline: TECShapeline;
begin

  if not assigned(map.shapes.lines[routes.ItemIndex]) then
    exit;

  Polyline := map.shapes.lines[routes.ItemIndex];

  if not assigned(Polyline) then
    exit;

  map.shapes.markers[idMarker].setPosition(Polyline.Path[ValueIndex].latitude,
    Polyline.Path[ValueIndex].longitude);

  map.Panto(map.shapes.markers[idMarker].latitude, map.shapes.markers[idMarker]
      .longitude);

  Chart.title.Text[0] := map.address;
  Chart.refresh;
end;

procedure TFDemoNativeRoute.btAbortAltClick(Sender: TObject);
begin
  btAbortAlt.tag := -1;
end;

end.
