unit UMainNativeVelib;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,uecGraphics,
  Dialogs, ECMaps, ExtCtrls,UECJCVelib, StdCtrls,uecmaputil, uecNativeMapControl,
  uecGeoLocalise,uecnativeshape;

type
  TFormNativeVelib = class(TForm)
    Panel1: TPanel;
    Label1: TLabel;
    cities: TComboBox;
    Label2: TLabel;
    Commercial: TLabel;
    Panel2: TPanel;
    infos: TMemo;
    stations: TComboBox;

    Panel3: TPanel;
    map: TECNativeMap;


    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure citiesChange(Sender: TObject);
    procedure mapOverlayClick(sender: TObject; const Index: Integer;
      const dLatitude, dLongitude: Double; const OverlayType: TOverlayType);
    procedure stationsChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure mapMapClick(sender: TObject; const Lat, Lng: Double);
    procedure mapShapeClick(sender: TObject; const item: TECShape);
    procedure FormActivate(Sender: TObject);
  private
    { Dclarations prives }
   FECVelib         : TECVelib;
   FStationSelected : TECShapePoi;



   procedure SelectStation(const index:integer);

   procedure doOwnerDrawPOI(const canvas:TECCanvas;var Rect:TRect;item:TECShape) ;

  public
    { Dclarations publiques }
  end;

var
  FormNativeVelib: TFormNativeVelib;

implementation

{$R *.dfm}

const NORMAL_STATION_COLOR = $00FF8000;
      SELECT_STATION_COLOR = $004848FF;
      WEBDINGS_BIKE        = 'b';

procedure TFormNativeVelib.citiesChange(Sender: TObject);
var i:integer;
    id:integer;
    Station : TECVelibStation;
    Poi     : TECShapePOI;
begin


 if assigned(FECVelib) and assigned(map) and
 assigned(FECVelib.Contracts[cities.ItemIndex]) then
 begin

  Infos.Lines.clear;




  Commercial.caption := FECVelib.Contracts[cities.ItemIndex].CommercialName;

  // find all stations in citie
  FECVelib.getContractStations(Cities.text) ;

  if FECVelib.Stations.Count=0 then
  begin
    ShowMessage('Sorry no station in '+Cities.text);
    exit;
  end;

  map.BeginUpdate;

  Stations.Items.BeginUpdate;

  map.shapes.Pois.clear;
  Stations.Items.clear;


  // add all stations
 for i := 0 to FECVelib.Stations.Count - 1 do
 begin

   Station := FECVelib.Stations[i];

   id := map.shapes.Pois.add(Station.latitude,Station.Longitude);

   Poi := map.shapes.Pois[id];

   Stations.Items.add(Station.Name);

   Poi.hint    := Station.Name;

   if assigned(map.shapes.pois.OnOwnerDraw) then
   begin
    Poi.poishape := poiOwnerDraw;
   end else begin
             Poi.POIShape      := poiStar;
            end;

   Poi.Width      := 25;
   Poi.Height     := 25;

   Poi.Color      := NORMAL_STATION_COLOR;
   Poi.HoverColor := SELECT_STATION_COLOR;

 end;

 Stations.Items.EndUpdate;
 map.EndUpdate;

  // show alls stations
  map.fitBounds(FECVelib.Stations.SouthWestLatitude,FECVelib.Stations.SouthWestLongitude,
                FECVelib.Stations.NorthEastLatitude,FECVelib.Stations.NorthEastLongitude);

 end;

end;

procedure TFormNativeVelib.FormActivate(Sender: TObject);
var i:integer;
begin

 if assigned(FECVelib) then    exit;

 // class for work with Velib api
  FECVelib := TECVelib.create;



  // get all cities where velib exist

  FECVelib.getContracts;

  for i := 0 to FECVelib.ContractsCount - 1 do
   Cities.items.add(FECVelib.Contracts[i].Name);



 Cities.ItemIndex := 0;
   citiesChange(nil);

end;

procedure TFormNativeVelib.FormClose(Sender: TObject; var Action: TCloseAction);
begin
 Action := caFree;

 FECVelib.free;
end;

procedure TFormNativeVelib.FormCreate(Sender: TObject);
var i:integer;
begin
   map.MouseWheelZoom := true;
   map.LocalCache := extractfilepath(application.exename)+'cache';

   if screen.fonts.indexof('WebDings')>-1 then
  map.shapes.pois.OnOwnerDraw := doOwnerDrawPOI;

  map.TileServer := tsOpenCycleMap;



end;

procedure TFormNativeVelib.mapMapClick(sender: TObject; const Lat, Lng: Double);
var station  : TECVelibStation;
    _lat,_lng:double;
    rp : TECRoutePath;
begin
 station := FECVelib.Stations.FindNearestStation(Lat,Lng) ;

 if not assigned(station) then exit;


 map.shapes.lines.clear;

 _lat := lat;
 _lng := lng;
 map.GeoLocalise.AlignLatLngToRoute(_lat,_lng);

 rp := map.GetRoutePathFrom([_lat,_lng,station.Latitude,station.Longitude]);

 if assigned(rp) and (rp.Distance>0) then
  map.AddRoute(rp)
 else showmessage('Route not found !');
end;

// owner draw poi, here transparancy text
procedure TFormNativeVelib.doOwnerDrawPOI(const canvas:TECCanvas;var Rect:TRect;item:TECShape) ;
begin

  canvas.Font.Name := 'WebDings';
  canvas.font.Size := 24;

  canvas.brush.Style := bsClear;

  if item.Hover then
     canvas.font.color := item.HoverColor
    else
     canvas.font.color := item.color;

  canvas.font.style := [fsBold];

  item.Width  := canvas.TextWidth(WEBDINGS_BIKE) ;
  item.height := canvas.TextHeight(WEBDINGS_BIKE) ;

  item.x := item.x - (item.Width  div 2);
  item.y := item.y - (item.height  div 2);




  canvas.TextOut(item.x,item.y,WEBDINGS_BIKE);

end;


procedure TFormNativeVelib.SelectStation(const index:integer);
var Station : TECVelibStation;
 latSW, lngSW, latNE, lngNE:double;
begin


  if assigned(FStationSelected) then
   FStationSelected.color := NORMAL_STATION_COLOR;

  Station :=  FECVelib.Stations[index];

  if not assigned(Station) then  exit;

  if Stations.ItemIndex<>index then
  begin
    Stations.OnChange  := nil;
    Stations.ItemIndex := index;
    Stations.OnChange  := stationsChange
  end;


  infos.lines.clear;

  infos.lines.add(Station.Address+#13#10) ;
  infos.lines.add('Available bikes : '+inttostr(Station.Available_bikes)+#13#10 );
  infos.lines.add('Available bikes stands: '+inttostr(Station.Available_bike_stands) );

  //  0.5km zoom around the selected station

  boundingCoordinates(Station.Latitude,Station.Longitude, 0.5, latSW, lngSW, latNE, lngNE);

  map.Bounds(latSW, lngSW, latNE, lngNE);

  FStationSelected       := map.shapes.Pois[index];
  FStationSelected.color := SELECT_STATION_COLOR;


end;


procedure TFormNativeVelib.mapOverlayClick(sender: TObject; const Index: Integer;
  const dLatitude, dLongitude: Double; const OverlayType: TOverlayType);

begin

  if OverlayType<>ovPoi then  exit;





end;

procedure TFormNativeVelib.mapShapeClick(sender: TObject; const item: TECShape);
begin
 SelectStation(item.indexof);
end;

procedure TFormNativeVelib.stationsChange(Sender: TObject);
begin

 SelectStation(Stations.itemindex);

end;

end.
