unit UMainOverlay;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ECMaps, ExtCtrls, Buttons, StdCtrls, Spin, ExtDlgs, ComCtrls, Menus;

type
  TFormOverlayDemo = class(TForm)
    pn_Maps: TPanel;
    OpenImage: TOpenPictureDialog;
    ColorDialog: TColorDialog;
    Pages: TPageControl;
    TabMap: TTabSheet;
    Tab3D: TTabSheet;
    Earth: TECMap;
    pn_Menu: TPanel;
    Bar_Menu: TPanel;
    SpeedMarker: TSpeedButton;
    SpeedPolyline: TSpeedButton;
    SpeedPolygone: TSpeedButton;
    SpeedCircle: TSpeedButton;
    SpeedRectangle: TSpeedButton;
    bar_overlay: TPanel;
    bar_overlay_new: TPanel;
    btNew: TBitBtn;
    bar_marker: TPanel;
    pnOpenImage: TPanel;
    btOpenPicture: TButton;
    bar_color: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    ColorBorder: TPanel;
    OpacityBorder: TSpinEdit;
    WeightBorder: TSpinEdit;
    bar_fillcolor: TPanel;
    Label4: TLabel;
    Label5: TLabel;
    ColorFond: TPanel;
    OpacityFond: TSpinEdit;
    bar_Points: TPanel;
    pnMetric: TPanel;
    Panel1: TPanel;
    mmMetrics: TMemo;
    pnPoints: TPanel;
    pnActionPoint: TPanel;
    btAltitude: TButton;
    Panel2: TPanel;
    mmPoints: TMemo;
    cbMarkerImage: TComboBox;
    Label6: TLabel;
    ckInfoWindow: TCheckBox;
    speedLoadMap: TSpeedButton;
    SpeedSaveMap: TSpeedButton;
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    pn_latlng: TPanel;
    latlng: TLabel;
    pn_adress: TPanel;
    cbAdress: TComboBox;
    Label7: TLabel;
    SpeedSaveKml: TSpeedButton;
    SpeedOverlay: TSpeedButton;
    ckalignToRoute: TCheckBox;
    PopupMenu1: TPopupMenu;
    pncarte: TPanel;
    lbCarte: TLabel;
    cbCarte: TComboBox;
    ckGoogle: TRadioButton;
    ckCloudmade: TRadioButton;
    map: TECMap;

    procedure EarthLoad(Sender: TObject);
    procedure EarthEarthViewInit(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure SpeedMarkerClick(Sender: TObject);
    procedure bar_markerResize(Sender: TObject);
    procedure btOpenPictureClick(Sender: TObject);
    procedure MapMapClick(sender: TObject; const dLatitude,
      dLongitude: Double);
    procedure MapOverlayChange(sender: TObject; const Index: Integer;
      const OverlayType: TOverlayType);
    procedure btNewClick(Sender: TObject);
    procedure MapEditOverlayPointClick(sender: TObject;
      const Index: Integer);
    procedure btAltitudeClick(Sender: TObject);
    procedure ColorBorderClick(Sender: TObject);
    procedure WeightBorderChange(Sender: TObject);
    procedure MapEditOverlayPointRClick(sender: TObject;
      const Index: Integer);
    procedure MapOverlayRightClick(sender: TObject; const Index: Integer;
      const dLatitude, dLongitude: Double;
      const OverlayType: TOverlayType);
    procedure MapMarkerClick(sender: TObject; const Index: Integer;
      const dLatitude, dLongitude: Double);
    procedure MapMarkerRightClick(sender: TObject; const Index: Integer;
      const dLatitude, dLongitude: Double);
    procedure MapMarkerDragEnd(sender: TObject; const Index: Integer;
      const dLatitude, dLongitude: Double);
    procedure MapLoad(Sender: TObject);
    procedure PagesChanging(Sender: TObject; var AllowChange: Boolean);
    procedure cbMarkerImageChange(Sender: TObject);
    procedure speedLoadMapClick(Sender: TObject);
    procedure SpeedSaveMapClick(Sender: TObject);
    procedure MapMapMouseMove(sender: TObject; const dLatitude,
      dLongitude: Double);
    procedure cbAdressKeyPress(Sender: TObject; var Key: Char);
    procedure MapAdress(sender: TObject; const sAdresses: String;
      var Index: Integer);
    procedure pn_adressResize(Sender: TObject);
    procedure cbAdressClick(Sender: TObject);
    procedure SpeedSaveKmlClick(Sender: TObject);
    procedure MapOverlayPathChange(sender: TObject; const Index: Integer;
      const OverlayType: TOverlayType; const PathIndex: Integer);
    procedure cbCarteChange(Sender: TObject);
    procedure ckgoogleClick(Sender: TObject);
    procedure ckcloudMadeClick(Sender: TObject);
    procedure MapAfterChangeMapApi(Sender: TObject);
    procedure editoverlayClick(Sender: TObject);
    procedure mapMapDragEnd(Sender: TObject);
    procedure mapOverlayClick(sender: TObject; const Index: Integer;
      const dLatitude, dLongitude: Double; const OverlayType: TOverlayType);
    procedure ckChromiumClick(Sender: TObject);
    procedure mapAfterChangeToTxt(Sender: TObject);
    procedure ckQuestClick(Sender: TObject);
  private
    { Dclarations prives }
    FOverlay : TOverlayType;
    FPoints  : TLatLngList;

    procedure setOverlay(const value:TOverlayType);

    procedure doUpdateMetricInfo;
    procedure doUpdateOverlay;
    procedure doUpdateOverlayInfo;


  public
    { Dclarations publiques }

    property Overlay : TOverlayType read FOverlay write setOverlay;
    property Points  : TLatLngList  read FPoints  write FPoints;
  end;

var
  FormOverlayDemo: TFormOverlayDemo;

implementation

{$R *.DFM}

const _CR          = #13#10;
      _PERIMETER   = 'Perimeter : %4.2f Km'+_CR;
      _SURFACE     = 'Surface   : %4.2f Km'+_CR;
      _RADIUS      = 'Radius    : %4.2f Km'+_CR;
      _CIRCUM      = 'Circum.   : %4.2f Km'+_CR;
      _LAT         = 'lat : %2.5f ';
      _LNG         = 'lnt :  %2.5f ';
      _LAT_LNG     = _LAT+_CR+_LNG;
      _LAT_LNG_ALT = _LAT+_LNG+'alt : %2.0f m'+_CR;



procedure TFormOverlayDemo.FormCreate(Sender: TObject);
begin
 speedMarker.tag    := integer(ovMarker);
 speedPolyline.tag  := integer(ovLine);
 speedPolygone.tag  := integer(ovPolygone);
 speedCircle.tag    := integer(ovCircle);
 speedRectangle.tag := integer(ovRectangle);
 speedOverlay.tag   := integer(ovGroundOverlay);

 pn_adressResize(nil);
end;



procedure TFormOverlayDemo.bar_markerResize(Sender: TObject);
begin
 cbMarkerImage.width := pnOpenImage.left-cbMarkerImage.left-7;
end;

procedure TFormOverlayDemo.pn_adressResize(Sender: TObject);
begin
 cbAdress.Width := pnCarte.left - 7 - cbAdress.left;
 //cbCarte. Width := pn_Adress.width - 7- cbCarte.left - 80;
end;

procedure TFormOverlayDemo.ckQuestClick(Sender: TObject);
begin
 //if ckQuest.checked then
 //  map.MapAPI := apiOpenMapQuest;
end;

procedure TFormOverlayDemo.cbAdressClick(Sender: TObject);
var lat,lng : double;
    i       : integer;
begin

 i := cbAdress.ItemIndex;

 if (i>-1) and (i<map.Adresses.count) then
  map.SetCenter(map.Adresses.latitude[i],map.Adresses.longitude[i])
  else
 if map.GetLatLngFromAdress(cbAdress.text,lat,lng) then
   map.SetCenter(lat,lng);
end;


{*
  Load map from txt file
}
procedure TFormOverlayDemo.speedLoadMapClick(Sender: TObject);
begin
 if OpenDialog.execute then
   map.LoadFromFile(OpenDialog.filename);
end;

{*

 save map to txt file
}
procedure TFormOverlayDemo.SpeedSaveMapClick(Sender: TObject);
begin
 SaveDialog.DefaultExt := 'txt';
 SaveDialog.Filter     := '*.'+SaveDialog.DefaultExt;

 if SaveDialog.execute then
  map.SaveToFile(SaveDialog.filename);
end;

{*

 Save map to kml file

*}
procedure TFormOverlayDemo.SpeedSaveKmlClick(Sender: TObject);
begin

 SaveDialog.DefaultExt := 'kml';
 SaveDialog.Filter     := '*.'+SaveDialog.DefaultExt;

 if SaveDialog.execute then
  map.SaveToKmlFile(SaveDialog.filename);
end;



{*
   select a type of overlay

   the Options Bar is updated according to the choice

   @param value overlay type


}
procedure TFormOverlayDemo.setOverlay(const value:TOverlayType);
begin

  if FOverlay=value then exit;
  
  FOverlay := value;

  bar_points.visible       := true;

  map.DraggableCursor      := 'CrossHair';



  case FOverlay of

    ovGroundOverlay,
    ovMarker :  begin
                 bar_marker.width   := bar_overlay.width div 2;
                 bar_marker.visible := true;

                 bar_overlay_new.visible  := false;
                 bar_color.visible        := false;
                 bar_fillcolor.visible    := false;
                 pnPoints.visible         := false;

                 speedMarker.down         := FOverlay = ovMarker;
                 speedOverlay.down        := FOverlay = ovGroundOverlay;

                end;

    ovLine   :   begin
                  bar_color.visible := true;
                  bar_overlay_new.visible  := true;
                  pnPoints.visible         := true;

                  bar_marker.visible       := false;
                  bar_fillcolor.visible    := false;

                  speedPolyline.down       := true;
                 end;

    ovPolygone,
    ovCircle,
    ovRectangle : begin
                  bar_color.visible        := true;
                  bar_fillcolor.visible    := true;
                  bar_overlay_new.visible  := FOverlay=ovPolygone;
                  pnPoints.visible         := FOverlay=ovPolygone;
                  bar_color.left           := 0;

                  bar_marker.visible       := false;

                  speedPolygone.down       := FOverlay=ovPolygone;
                  speedcircle.down         := FOverlay=ovCircle;
                  speedRectangle.down      := FOverlay=ovRectangle;

                 end;

    ovNone   :   begin
                  map.DraggableCursor      := 'Default';
                  bar_color.visible        := false;
                  bar_fillcolor.visible    := false;
                  bar_overlay_new.visible  := false;
                  pnPoints.visible         := false;
                  bar_marker.visible       := false;
                  bar_points.visible       := false;
                  mmPoints .Lines.Text     := '';
                  mmMetrics.Lines.Text     := '';
                  speedMarker.down         := false;
                  speedPolyline.down       := false;
                  speedPolygone.down       := false;
                  speedcircle.down         := false;
                  speedRectangle.down      := false;
                  speedOverlay.down        := false;
                  map.EditOverlay          := nil;
                 end;

  end;

  bar_overlay_new.left := 0;
  pnPoints.width       := 380;
  bar_points.left      := 1000;

end;


{*
   Display geographic information based on the active overlay
}
procedure TFormOverlayDemo.doUpdateMetricInfo;
var i    : integer;
    dataPoint,
    dataMetric : string;
begin



  dataPoint := '';
  dataMetric:= '';


  case Overlay of

   ovMarker  : begin
                if assigned(map.EditMarker) then
                begin
                 dataMetric := format(_LAT_LNG_ALT,[map.EditMarker.latitude,
                                                                    map.EditMarker.longitude,
                                                                    map.EditMarker.altitude])
                     + _CR + map.EditMarker.Adress;
                end;
               end;

   ovLine,
   ovPolygone : begin
                  if assigned(Points) then
                  begin
                    for i:=0 to Points.count-1 do
                       dataPoint := dataPoint
                     + format(_LAT_LNG_ALT,[Points[i].latitude,
                                                                    Points[i].longitude,
                                                                    Points[i].altitude]);


                      dataMetric := format(_PERIMETER,[Points.distance/1000]);

                      if (overlay = ovPolygone)   and
                         assigned(map.EditOverlay)  then

                        dataMetric := dataMetric +
                                      format(_SURFACE,[map.Polygones[map.EditOverlay.id].area]);
                    end;


                 end;

   ovCircle    : if assigned(map.EditOverlay) then

                 begin
                    dataMetric := format(_CIRCUM, [map.Circles[map.EditOverlay.id].distance/1000]) +
                                  format(_RADIUS, [map.Circles[map.EditOverlay.id].radius/1000]) +
                                  format(_SURFACE,[map.Circles[map.EditOverlay.id].area]);
                  end;

   ovRectangle : if assigned(map.EditOverlay) then

                 begin
                  dataMetric := format(_PERIMETER,[map.Rectangles[map.EditOverlay.id].distance/1000]) +
                                format(_SURFACE,  [map.Rectangles[map.EditOverlay.id].area]);
                end;

   ovGroundOverlay : if assigned(map.EditOverlay) then

                 begin
                  dataMetric := format(_PERIMETER,[map.GroundOverlays[map.EditOverlay.id].distance/1000]) +
                                format(_SURFACE,  [map.GroundOverlays[map.EditOverlay.id].area]);
                end;


  end;

  mmPoints .Lines.Text := dataPoint;
  mmMetrics.Lines.Text := dataMetric;



end;


{*
   onload event of the component earth
}
procedure TFormOverlayDemo.EarthLoad(Sender: TObject);
begin
  earth.EarthView.Visible:=true;
end;




procedure TFormOverlayDemo.editoverlayClick(Sender: TObject);
begin
end;

{*
  OnEarthViewInit event of the component earth

  triggered when the 3d plugin is ok
}
procedure TFormOverlayDemo.EarthEarthViewInit(Sender: TObject);
begin
 earth.EarthView.FlyToSpeed := 6.0;
 earth.EarthView.Tilt := 0;
 earth.EarthView.Range := 500;
 earth.EarthView.NavigationControlVisibility :=  ncvShow;
 earth.EarthView.LookAt(map.latitude,map.longitude);
 // load kml from map
 Earth.EarthView.ToKml := Map.toKml;
 
end;


{*
   click on overlay button
}
procedure TFormOverlayDemo.SpeedMarkerClick(Sender: TObject);
begin

  if not (sender is TSpeedButton) then exit;


  btNew.hint := 'Add '+TSpeedButton(sender).hint;

  map.EditOverlay  := nil;


  if TSpeedButton(sender).down then
   Overlay := TOverlayType(TSpeedButton(sender).tag)
  else
   OVerlay := ovNone;

end;






{*
   select a local image for a marker
}
procedure TFormOverlayDemo.btOpenPictureClick(Sender: TObject);
begin
 if OpenImage.execute then
 begin
   cbMarkerImage.text := OpenImage.filename;
   if cbMarkerImage.Items.indexof(cbMarkerImage.text)=-1 then
    cbMarkerImage.Items.add(cbMarkerImage.text);

  if assigned(map.EditMarker) then
   map.EditMarker.Icon := cbMarkerImage.text;

 end;
end;


{*
   OnMapClick event for componant map


}
procedure TFormOverlayDemo.MapMapClick(sender: TObject; const dLatitude,
  dLongitude: Double);

  var Lat,Lng:double;
var id:integer;
begin


 Lat := dLatitude;
 Lng := dLongitude;


 if ckAlignToRoute.checked then
  map.AlignLatLngToRoute(Lat,Lng,tmDriving);


 map.OnOverlayChange := nil;

 case Overlay of

  ovMarker : begin

               id := map.Markers.add(Lat,Lng);

               map.editmarker := map.Markers[id];

               if assigned(map.editmarker) then
               begin
                 map.editmarker.Icon      := cbMarkerImage.text;
                 map.editmarker.Draggable := true;


               end;

               doUpdateMetricInfo;

               if ckInfoWindow.Checked then
                map.Markers[id].InfoWindow := map.InfoWindows.Add(mmMetrics.Text);
             end;

  ovGroundOverlay : begin
                      id := map.GroundOverlays.Add(cbMarkerImage.text,Lat,Lng) ;
                      map.editOverlay := map.GroundOverlays[id];

                     doUpdateMetricInfo;

                      if ckInfoWindow.Checked then
                    map.GroundOverlays[id].infoWindow := map.InfoWindows.Add(mmMetrics.Text);


  end;

  ovLine   :

               if map.editOverlay=nil then
               begin

                  id := map.Polylines.Add;

                  map.editOverlay := map.Polylines[id];

                  doUpdateOverlay;

                  doUpdateMetricInfo;

                  if ckInfoWindow.Checked then
                    map.Polylines[id].infoWindow := map.InfoWindows.Add(mmMetrics.Text);
               end;



  ovPolygone : if map.editOverlay=nil then
               begin

                  id := map.Polygones.Add;

                  map.editOverlay := map.Polygones[id];

                  doUpdateOverlay;
                  doUpdateMetricInfo;

                  if ckInfoWindow.Checked then
                    map.Polygones[id].infoWindow := map.InfoWindows.Add(mmMetrics.Text);
               end;

  ovCircle  :
              begin

                id := map.Circles.add(Lat,Lng);

                map.editOverlay := map.circles[id];

                doUpdateOverlay;
                doUpdateMetricInfo;

                if ckInfoWindow.Checked then
                  map.Circles[id].infoWindow := map.InfoWindows.Add(mmMetrics.Text);
              end;

  ovRectangle :
                 begin

                  id := map.Rectangles.add(Lat,Lng);

                  map.editOverlay := map.Rectangles[id];

                  doUpdateOverlay;
                  doUpdateMetricInfo;

                  if ckInfoWindow.Checked then
                    map.Rectangles[id].infoWindow := map.InfoWindows.Add(mmMetrics.Text);
                 end;
 end;



 if assigned(map.EditOverlay) then
                map.EditOverlay.add(Lat,Lng);

 map.OnOverlayChange := MapOverlayChange;

 map.editoverlay := map.EditOverlay;

end;


procedure TFormOverlayDemo.mapMapDragEnd(Sender: TObject);
begin

end;

{*
   OnOverlayChange event

   @param index index of overlay
   @param OverlayType type of overlay
}
procedure TFormOverlayDemo.MapOverlayChange(sender: TObject;
  const Index: Integer; const OverlayType: TOverlayType);
begin

  Points := nil;

   case OverlayType of

     ovLine      : Points := map.Polylines[index].path;
     ovPolygone  : Points := map.Polygones[index].path;

   end;


  Overlay := OverlayType;


  doUpdateMetricInfo;
  doUpdateOverlayInfo;

  if assigned(map.editOverlay) and
    (map.editOverlay.InfoWindow>-1) and
    (ckInfoWindow.Checked) then
  map.InfoWindows[map.editOverlay.InfoWindow].Content := mmMetrics.Text;


end;



procedure TFormOverlayDemo.mapOverlayClick(sender: TObject;
  const Index: Integer; const dLatitude, dLongitude: Double;
  const OverlayType: TOverlayType);
begin
 Points := nil;

   case OverlayType of

     ovLine      : begin
                    Points := map.Polylines[index].path;
     end;

     ovPolygone  : Points := map.Polygones[index].path;

   end;




  Overlay := OverlayType;


  doUpdateMetricInfo;
  doUpdateOverlayInfo;


end;

procedure TFormOverlayDemo.MapOverlayPathChange(sender: TObject;
  const Index: Integer; const OverlayType: TOverlayType;
  const PathIndex: Integer);

  var lat,lng : double;
      Path  : TLatLngList;
begin
 if not ckaligntoroute.checked then exit;

 case OverlayType of

   ovLine     : Path := map.Polylines[index].Path;
   ovPolygone : Path := map.Polygones[index].Path;


 end;

 if not assigned(LatLng) then exit;


 lat := Path[pathindex].Latitude;
 lng := Path[pathindex].Longitude;
 if map.AlignLatLngToRoute(lat,lng,tmDriving) then
 begin
   Path.BeginUpdate;
    Path[pathindex].Latitude := lat;
    Path[pathindex].Longitude:= lng;
   Path.EndUpdate;
 end;

end;

procedure TFormOverlayDemo.btNewClick(Sender: TObject);
begin

 map.EditOverlay := nil;


 case Overlay Of

   ovLine     :  map.editOverlay := map.Polylines[map.Polylines.Add];

   ovPolygone : map.editOverlay := map.Polygones[map.Polygones.Add];

 end;

 // add new overlay on center of map
 if assigned(map.EditOverlay) then
                map.EditOverlay.add(map.Latitude,map.Longitude);


 doUpdateOverlay;

end;

procedure TFormOverlayDemo.MapEditOverlayPointClick(sender: TObject;
  const Index: Integer);
begin
 with mmPoints do
 begin
  SelStart := Perform(EM_LINEINDEX, index, 0);
  Perform(EM_SCROLLCARET, 0, 0);
  SelLength:= length(mmPoints.lines[index]);
 end;
end;


procedure TFormOverlayDemo.btAltitudeClick(Sender: TObject);
begin

  if not assigned(Points) then exit;
  
  Points.getAltitudes(nil);

  doUpdateMetricInfo;

end;

procedure TFormOverlayDemo.ColorBorderClick(Sender: TObject);
begin
 if ColorDialog.execute then
 begin
  (Sender As TPanel).color := ColorDialog.color;

  doUpdateOverlay;
 end;
end;


procedure TFormOverlayDemo.WeightBorderChange(Sender: TObject);
begin
 doUpdateOverlay;
end;


procedure TFormOverlayDemo.doUpdateOverlay;
begin

   if (map.EditOverlay=nil) then exit;

   case map.EditOverlay.OverlayType of

    ovLine :     begin
                   map.Polylines[map.EditOverlay.Id].color   := ColorBorder.color;
                   map.Polylines[map.EditOverlay.Id].opacity := OpacityBorder.Value / 100;
                   map.Polylines[map.EditOverlay.Id].weight  := WeightBorder.value;
                   map.Polylines[map.EditOverlay.Id].reDraw;
                 end;

    ovPolygone : begin
                   map.Polygones[map.EditOverlay.Id].color       := ColorBorder.color;
                   map.Polygones[map.EditOverlay.Id].opacity     := OpacityBorder.Value / 100;
                   map.Polygones[map.EditOverlay.Id].weight      := WeightBorder.value;
                   map.Polygones[map.EditOverlay.Id].fillcolor   := Colorfond.color;
                   map.Polygones[map.EditOverlay.Id].fillopacity := OpacityFond.Value / 100;
                   map.Polygones[map.EditOverlay.Id].redraw;
                 end;

    ovCircle  : begin
                   map.Circles[map.EditOverlay.Id].color       := ColorBorder.color;
                   map.Circles[map.EditOverlay.Id].opacity     := OpacityBorder.Value / 100;
                   map.Circles[map.EditOverlay.Id].weight      := WeightBorder.value;
                   map.Circles[map.EditOverlay.Id].fillcolor   := Colorfond.color;
                   map.Circles[map.EditOverlay.Id].fillopacity := OpacityFond.Value / 100;
                   map.Circles[map.EditOverlay.Id].redraw;
                 end;

    ovRectangle : begin
                   map.Rectangles[map.EditOverlay.Id].color       := ColorBorder.color;
                   map.Rectangles[map.EditOverlay.Id].opacity     := OpacityBorder.Value / 100;
                   map.Rectangles[map.EditOverlay.Id].weight      := WeightBorder.value;
                   map.Rectangles[map.EditOverlay.Id].fillcolor   := Colorfond.color;
                   map.Rectangles[map.EditOverlay.Id].fillopacity := OpacityFond.Value / 100;
                   map.Rectangles[map.EditOverlay.Id].redraw;
                 end;


   end;

end;


procedure TFormOverlayDemo.doUpdateOverlayInfo;
begin

   if (map.EditOverlay=nil) then exit;

   OpacityBorder.OnChange := nil;
   WeightBorder .OnChange := nil;
   OpacityFond  .OnChange := nil;

   case Overlay of

    ovLine :     begin
                   ColorBorder.color    := map.Polylines[map.EditOverlay.Id].color;
                   OpacityBorder.Value  := round(map.Polylines[map.EditOverlay.Id].opacity * 100);
                   WeightBorder.value   := map.Polylines[map.EditOverlay.Id].weight;
                 end;

    ovPolygone : begin
                   ColorBorder.color    := map.Polygones[map.EditOverlay.Id].color;
                   OpacityBorder.Value  := round(map.Polygones[map.EditOverlay.Id].opacity * 100);
                   WeightBorder.value   := map.Polygones[map.EditOverlay.Id].weight;
                   Colorfond.color      := map.Polygones[map.EditOverlay.Id].fillcolor;
                   OpacityFond.Value    := round(map.Polygones[map.EditOverlay.Id].fillopacity * 100);
                 end;

    ovCircle : begin
                   ColorBorder.color    := map.Circles[map.EditOverlay.Id].color;
                   OpacityBorder.Value  := round(map.Circles[map.EditOverlay.Id].opacity * 100);
                   WeightBorder.value   := map.Circles[map.EditOverlay.Id].weight;
                   Colorfond.color      := map.Circles[map.EditOverlay.Id].fillcolor;
                   OpacityFond.Value    := round(map.Circles[map.EditOverlay.Id].fillopacity * 100);
                 end;

    ovRectangle : begin
                   ColorBorder.color    := map.Rectangles[map.EditOverlay.Id].color;
                   OpacityBorder.Value  := round(map.Rectangles[map.EditOverlay.Id].opacity * 100);
                   WeightBorder.value   := map.Rectangles[map.EditOverlay.Id].weight;
                   Colorfond.color      := map.Rectangles[map.EditOverlay.Id].fillcolor;
                   OpacityFond.Value    := round(map.Rectangles[map.EditOverlay.Id].fillopacity * 100);
                 end;


   end;


   OpacityBorder.OnChange := WeightBorderChange;
   WeightBorder .OnChange := WeightBorderChange;
   OpacityFond  .OnChange := WeightBorderChange;

end;











procedure TFormOverlayDemo.MapEditOverlayPointRClick(sender: TObject;
  const Index: Integer);
begin
 map.EditOverlay.Delete(index);
end;

procedure TFormOverlayDemo.MapOverlayRightClick(sender: TObject;
  const Index: Integer; const dLatitude, dLongitude: Double;
  const OverlayType: TOverlayType);
begin


 case OverlayType of

  ovLine      : map.Polylines.Delete(index);
  ovPolygone  : map.Polygones.Delete(index);
  ovcircle    : map.Circles.Delete(index);
  ovRectangle : map.Rectangles.Delete(index);
  ovGroundOverlay : map.GroundOverlays.Delete(index);


 end;

 Overlay := ovnone;

end;

procedure TFormOverlayDemo.MapMarkerClick(sender: TObject;
  const Index: Integer; const dLatitude, dLongitude: Double);
begin
   overlay := ovMarker;
   cbMarkerImage.text := map.Markers[index].icon;
   doUpdateMetricInfo;
end;

procedure TFormOverlayDemo.MapMarkerRightClick(sender: TObject;
  const Index: Integer; const dLatitude, dLongitude: Double);
begin

 map.InfoWindows.Delete(map.Markers[index].InfoWindow);
 map.Markers.delete(index);
 Overlay := ovnone;

end;

procedure TFormOverlayDemo.MapMarkerDragEnd(sender: TObject;
  const Index: Integer; const dLatitude, dLongitude: Double);
begin
 Overlay := ovMarker;
 doUpdateMetricInfo;

 if (ckInfoWindow.Checked) and
    assigned(map.InfoWindows[map.Markers[index].InfoWindow]) then
   map.InfoWindows[map.Markers[index].InfoWindow].Content := mmMetrics.Text;

end;



procedure TFormOverlayDemo.MapLoad(Sender: TObject);
begin
 Overlay          := ovNone;
 Points           := nil;
 Bar_Menu.enabled := true;

 map.MobilesEnabled := true;

 cbCarte.Items.Text := map.MapTypes.Text;

 map.EditMode := true;

end;


procedure TFormOverlayDemo.PagesChanging(Sender: TObject;
  var AllowChange: Boolean);
begin
 if (Pages.ActivePage = TabMap) and assigned(Earth.EarthView) then
 begin
  Earth.EarthView.Clear;
  Earth.EarthView.ToKml     := Map.toKml;
  Earth.EarthView.LookAt(map.latitude, map.longitude);
 end

 else

 if (Pages.ActivePage = Tab3D) then
 begin
   map.SetCenter(earth.EarthView.Latitude,earth.EarthView.Longitude);
 end;

end;

procedure TFormOverlayDemo.cbMarkerImageChange(Sender: TObject);
begin
 if assigned(map.EditMarker) then
  map.EditMarker.Icon := cbMarkerImage.text
 else
  if assigned(map.EditOverlay) and
        (map.EditOverlay.OverlayType=ovGroundOverlay) then

     map.GroundOverlays[map.EditOverlay.Id].Url := cbMarkerImage.text;

end;




procedure TFormOverlayDemo.ckChromiumClick(Sender: TObject);
begin
(* if ckChromium.checked then
  map.DisplayBrowser := dbChromium
 else
  map.DisplayBrowser := dbIE;
 *)
end;

procedure TFormOverlayDemo.ckcloudMadeClick(Sender: TObject);
begin
 if ckCloudMade.checked then
   map.MapAPI := apiCloudmade;
end;

procedure TFormOverlayDemo.ckgoogleClick(Sender: TObject);
begin
 if ckGoogle.checked then
  map.MapAPI := apiGoogle;

end;

procedure TFormOverlayDemo.MapMapMouseMove(sender: TObject;
  const dLatitude, dLongitude: Double);
begin
 latlng.caption := format(_LAT_LNG,[dlatitude,dlongitude]);
end;

procedure TFormOverlayDemo.cbAdressKeyPress(Sender: TObject;
  var Key: Char);
begin
 if key=#13 then cbAdressClick(cbAdress);

end;

procedure TFormOverlayDemo.cbCarteChange(Sender: TObject);
begin
 map.MapTypeId := StrToMt(cbCarte.Text);
end;

procedure TFormOverlayDemo.MapAdress(sender: TObject;
  const sAdresses: String; var Index: Integer);
begin
 cbAdress.Items.Text := sAdresses;
 cbAdress.ItemIndex  := Index;
end;



procedure TFormOverlayDemo.MapAfterChangeMapApi(Sender: TObject);
begin
 cbCarte.Items.Text := map.MapTypes.Text;
end;

procedure TFormOverlayDemo.mapAfterChangeToTxt(Sender: TObject);
begin
// ckChromium.Checked := map.DisplayBrowser = dbChromium;

 case map.mapApi of

   apiGoogle     : ckGoogle.checked    := true;
   apiCloudMade  : ckCloudMade.checked := true;
            //  else ckQuest.checked  := true;
 end;

 map.EditMode := true;
end;

end.
